// 5648-C84, 5648-C83, (C) Copyright IBM Corporation, 1997, 2000
// All rights reserved. Licensed Materials Property of IBM
// Note to US Government users: Documentation related to restricted rights
// Use, duplication or disclosure is subject to restrictions set forth in GSA ADP Schedule with IBM Corp.
// This page may contain other proprietary notices and copyright information, the terms of which must be observed and followed.
//
// This program may be used, executed, copied, modified and distributed
// without royalty for the purpose of developing, using,
// marketing, or distributing.
package WebSphereSamples.Database;

import java.lang.*;
import java.util.*;
import java.sql.*;
import java.io.*;

/**
* This bean creates and populates the database.
*/

public class CreateDatabaseBean extends java.lang.Object implements java.io.Serializable {
	/**
	  * Class variable that indicates whether or not any instance of CreateDatabaseBean is creating a database
	  */
	private static boolean creatingDatabase = false;

	/**
	  * Class variable used to access critical section of code
	  */
	private static Object creatingDatabaseLock = new Object();

	/**
	  * Instance variable for the name of this class
	  */
	protected String className = getClass().getName();
	
	/**
	  * Instance variable for the driver property
	  */
	protected String driver = "";

	/**
	  * Instance variable for the url property
	  */
	protected String url = "";

	/**
	  * Instance variable for the userid property
	  */
	protected String userid = "";

	/**
	  * Instance variable for the password property
	  */
	protected String password = "";

	/**
	  * Instance variable for the sample question
	  */
	protected String question = "";

	/**
	  * Instance variable for an error string
	  */
	protected String error = "";
  /*****************************************************************************
  * Do the real work here
  * 
  */
  public void createDatabase() throws SQLException, IOException {
	Connection connectionToDatabase = getDBConnection();
	InputStream is = null;
	InputStreamReader isr = null;
	BufferedReader br = null;
	Vector tables = new Vector();
	String tableFile = "tables.txt";
	String table = null;
	String create = null;
	String drop = null;
	String line = null;
	String alias = null;
	String dropAlias = null;

	if (CreateDatabaseBean.creatingDatabase == true) {
		this.error = "Database is already being created";
		throw new java.sql.SQLException("Database is already being created");
	}  // end if

	synchronized(CreateDatabaseBean.creatingDatabaseLock) {
		CreateDatabaseBean.creatingDatabase = true;

		try {
			is = this.getClass().getResourceAsStream(tableFile);
			isr = new InputStreamReader(is);
			br = new BufferedReader(isr);
			executeSQLStatement(connectionToDatabase, "GRANT BINDADD ON DATABASE to USER wsdemo", true);
			executeSQLStatement(connectionToDatabase, "GRANT CONNECT ON DATABASE to USER wsdemo", true);
			executeSQLStatement(connectionToDatabase, "GRANT CREATETAB ON DATABASE to USER wsdemo", true);
			executeSQLStatement(connectionToDatabase, "GRANT DBADM ON DATABASE to USER wsdemo", true);

			while ((line = br.readLine()) != null) {
				StringTokenizer tokenizer = new StringTokenizer(line, "=");
				table = tokenizer.nextToken();
				create = tokenizer.nextToken();
				drop = "drop table " + table;
				alias = "create alias " + "wsdemo." + table + " for " + userid + "." + table;
				dropAlias = "drop alias wsdemo." + table;
				executeSQLStatement(connectionToDatabase, dropAlias, true);
				executeSQLStatement(connectionToDatabase, drop, true);
				executeSQLStatement(connectionToDatabase, create, true);
				executeSQLStatement(connectionToDatabase, alias, true);
				tables.addElement(table);
			}  // end while

			br.close();

			executeSQLStatement(connectionToDatabase, "drop alias " + "wsdemo.EMPLOYEE", true);
			executeSQLStatement(connectionToDatabase, "drop alias " + "wsdemo.DEPARTMENT", true);
			executeSQLStatement(connectionToDatabase, "create alias " + "wsdemo.EMPLOYEE for " + userid + ".EMPLOYEE", true);
			executeSQLStatement(connectionToDatabase, "create alias " + "wsdemo.DEPARTMENT for " + userid + ".DEPARTMENT", true);
			executeSQLStatement(connectionToDatabase, "insert into POLL values (1, 0, 0, '" + this.question + "')", true);

			for (Enumeration e = tables.elements(); e.hasMoreElements(); table = (String) e.nextElement()) {
				if ((is = this.getClass().getResourceAsStream(table + ".txt")) != null) {
					isr = new InputStreamReader(is);
					br = new BufferedReader(isr);

					while ((line = br.readLine()) != null) {
						StringTokenizer tokenizer = new StringTokenizer(line, ";");
						String sql = "insert into " + table + " values (" + tokenizer.nextToken();

						while (tokenizer.hasMoreTokens()) {
							sql += ", " + tokenizer.nextToken();
						}  // end while

						sql += ")";
						executeSQLStatement(connectionToDatabase, sql, true);
					}  // end while

					br.close();
				}  // end if
			}  // end for
		} finally {
			if (connectionToDatabase != null) {
				try {
					connectionToDatabase.close();
				} catch(Exception ex) {
					ex.printStackTrace(System.out);
				}

				connectionToDatabase = null;
			}

			CreateDatabaseBean.creatingDatabase = false;
		}  // end catch-finally
	}  // end synchronized block
  }  
  protected void executeSQLStatement(Connection connectionToDatabase, String stmt, boolean throwException)
  throws SQLException, IOException {
	java.sql.Statement sqlStatement = null;

	try {
		sqlStatement = connectionToDatabase.createStatement();

		if (connectionToDatabase.getAutoCommit() == true) {
			connectionToDatabase.setAutoCommit(false);
		}

		System.out.println(this.className + "  - Executing SQL :" + stmt);
		System.out.flush();
		sqlStatement.executeUpdate(stmt);
		connectionToDatabase.commit();
		sqlStatement.close();
		sqlStatement = null;
		System.out.flush();
	} catch(SQLException e) {
		if (sqlStatement != null) {
			try {
				sqlStatement.close();
				sqlStatement = null;
			} catch(Exception ex) {
				System.out.println(this.className + "  - Warning: an error occurred while attempting to close " + sqlStatement.getClass().getName() + " :  " + ex.toString());
				System.out.flush();
			}
		}

		// ignore the code for table already exists
		if ((e.getErrorCode() != -204) && (e.getErrorCode() != -601)) {
			System.out.println(this.className + "  - SQLException occurred during database setup");
			System.out.println(this.className + "  - Check that all of the database access parameters submitted are accurate.  (Note that the username and password are case sensitive.)");
			System.out.println(this.className + "  - SQLState=" + e.getSQLState() + " error code=" + e.getErrorCode());
			System.out.flush();
			error = e.getMessage();

			if (throwException) {
				throw e;
			}
		}
	}
  }            
  protected java.sql.Connection getDBConnection()
  throws SQLException {
	try {
		// Attempt to load database driver
		Class.forName(this.driver).newInstance();
	} catch(Exception ex) {
		String errorMessage = ex.toString();
		this.error = "Unable to load database driver";
		System.out.println(this.className + "  - Unable to load database driver  : " + errorMessage);
		System.out.flush();
		throw new java.sql.SQLException(this.className + "  - Unable to load database driver  : " + errorMessage);
	}

	try {
		// Now attempt to create a database connection
		return DriverManager.getConnection(this.url, this.userid, this.password);
	} catch(java.sql.SQLException sqlEx) {
		this.error = "Unable to get connection to database.  " + sqlEx.getMessage();
		System.out.println(this.className + "  - Unable to get connection to database  : " + sqlEx.toString());
		System.out.flush();

		throw sqlEx;
	}
  }              
  /*****************************************************************************
  * Get method for the error property
  * 
  * @param value the new value for the property
  */
  public String getError() {
	  return this.error;
  }    
  /*****************************************************************************
  * Set method for the driver property
  * 
  * @param value the new value for the property
  */
  public void setDriver(String value) {
	this.driver = value;
  }    
  /*****************************************************************************
  * Set method for the password property
  * 
  * @param value the new value for the property
  */
  public void setPassword(String value) {
	this.password = value;
  }    
  /*****************************************************************************
  * Set method for the question property
  * 
  * @param value the new value for the property
  */
  public void setQuestion(String value) {
	this.question = value;
  }    
  /*****************************************************************************
  * Set method for the url property
  * 
  * @param value the new value for the property
  */
  public void setUrl(String value) {
	this.url = value;
  }    
  /*****************************************************************************
  * Set method for the userid property
  * 
  * @param value the new value for the property
  */
  public void setUserid(String value) {
	this.userid = value;
  }    
}
